#include <SDL.h>
#include <SDL_image.h>
#include <iostream>

#include <godlibdefines.h>
#include <gp2x.h>
#include <sdltools.h>
#include <sdlttf.h>
#include <image.h>
#include <music.h>
#include <stdio.h>
#include <list>

#include "game.h"
#include "gfx.h"
#include "intro.h"
#include "filelist.h"
#include "filelistrandomizer.h"

using namespace std;

SDL_Surface *screen = NULL;
SDL_Joystick *joy = NULL;

FileListRandomizer *randomMusic;

void Terminate(void) {
	closeAudioSystem();
	SDL_Quit();
#ifdef GP2X
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#endif
}

void print_help() {
	cout << "usage: symbolica [-noaudio] [-random] <screenmode>" << endl << endl;
	cout << "-noaudio	Disable sound" << endl;
	cout << "-norandom/-r	Use predefined levels" << endl;
	cout << "-skipintro/-s	Skip intro screen" << endl << endl;
	cout << "Supported screenmodes:" << endl << endl;
	cout << "0 320x240" << endl;
	cout << "1 640x480" << endl;
	cout << "2 800x600" << endl;
	cout << endl;
	exit(0);
}

void config_dialog(int *useRandom, int *numberOfRandomLevels) {
	SDL_Surface *screen=SDL_GetVideoSurface();
	SDL_Rect blackRect;
	SDL_Event event;
	SDLTTF ttf(SHAREDIR"/"FONT,getSmallFontSize());
	ttf.setForegroundColor(0xff,0xff,0xff);
	int done=0;
	int active=0;
	int numberOfOptions=2;
	int keyVolDown=0;
	int keyVolUp=0;

	blackRect.x=getScreenCoordinate(5);
	blackRect.y=getScreenCoordinate(5);
	blackRect.w=screen->w-getScreenCoordinate(10);
	blackRect.h=screen->h-getScreenCoordinate(40);
	Image bg;
	bg.load(SHAREDIR"/data/gfx/"+intToString(getSizeID())+"/mainscreen.png");
	Image ball;
	ball.load(SHAREDIR"/data/gfx/"+intToString(getSizeID())+"/player0013.png",0);

	while (!done) {
		while (SDL_PollEvent (&event)) {
			switch (event.type) {
				case SDL_KEYDOWN:
				case SDL_JOYBUTTONDOWN:
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym==SDLK_RETURN) ||
							(event.type==SDL_KEYDOWN && event.key.keysym.sym==SDLK_ESCAPE) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_START) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_FB)
					   ) {
						done=1;
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym==SDLK_DOWN) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_DOWN)
					   ) {
						if (active<numberOfOptions-1) active++;
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym==SDLK_UP) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_UP)
					   ) {
						if (active>0) active--;
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym==SDLK_LEFT) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_LEFT)
					   ) {
						switch (active) {
							case 0:
								if (*useRandom==0) {
									*useRandom=1;
								} else {
									*useRandom=0;
								}
								break;
							case 1:
								if (*numberOfRandomLevels>5) *numberOfRandomLevels-=5;
								break;
						}
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym==SDLK_RIGHT) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_RIGHT)
					   ) {
						switch (active) {
							case 0:
								if (*useRandom==0) {
									*useRandom=1;
								} else {
									*useRandom=0;
								}
								break;
							case 1:
								*numberOfRandomLevels+=5;
								break;
						}
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym == SDLK_1) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_VOL_DOWN)
					   ) {
						keyVolDown=1;
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym == SDLK_2) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_VOL_UP)
					   ) {
						keyVolUp=1;
					}
					break;
				case SDL_KEYUP:
				case SDL_JOYBUTTONUP:
					if (
							(event.type==SDL_KEYUP && event.key.keysym.sym == SDLK_1) ||
							(event.type==SDL_JOYBUTTONUP && event.jbutton.button == VK_VOL_DOWN)
					   ) {
						keyVolDown=0;
					}
					if (
							(event.type==SDL_KEYUP && event.key.keysym.sym == SDLK_2) ||
							(event.type==SDL_JOYBUTTONUP && event.jbutton.button == VK_VOL_UP)
					   ) {
						keyVolUp=0;
					}
					break;
			}
		}
		sdlLock(screen);
		if (keyVolDown) {
			globalMusicVolume--;
			if (globalMusicVolume<0) globalMusicVolume=0;
			setGlobalVolume(globalMusicVolume);
		}
		if (keyVolUp) {
			globalMusicVolume++;
			if (globalMusicVolume>128) globalMusicVolume=128;
			setGlobalVolume(globalMusicVolume);
		}
		bg.blit(screen,0,0);
		SDL_FillRect(screen,&blackRect,0);

		ttf.render(screen, "OPTIONS", getScreenCoordinate(135), getScreenCoordinate(15));
		if (*useRandom) {
			ttf.render(screen, "Level order: random",getScreenCoordinate(25),getScreenCoordinate(40));
		} else {
			ttf.render(screen, "Level order: predefined",getScreenCoordinate(25),getScreenCoordinate(40));
		}
		ttf.render(screen, "Number of random levels: "+intToString(*numberOfRandomLevels),getScreenCoordinate(25),getScreenCoordinate(55));
		ball.blit(screen,getScreenCoordinate(5),getScreenCoordinate(active*15+40-4));

		sdlUnlock(screen);
		SDL_Flip(screen);

		SDL_Delay(25);
	}
}

int main (int argc, char **argv) {
	int done=0;
	int i;
	int useaudio=1;
	int userandom=1;
	int numberOfRandomLevels=50;
	int skipintro=0;
	int knownparam;

	for (i=1;i<argc;i++) {
		knownparam=0;
		if (
				(strcmp(argv[i],"-h")==0) ||
				(strcmp(argv[i],"-?")==0) ||
				(strcmp(argv[i],"?")==0) ||
				(strcmp(argv[i],"-help")==0) ||
				(strcmp(argv[i],"--help")==0)
		) {
			print_help();
		}

	       	if (
				(strcmp(argv[i],"-noaudio")==0) ||
				(strcmp(argv[i],"-nosound")==0)
		   ) 	{
			useaudio=0;
			knownparam=1;
		}
	       	if (
				(strcmp(argv[i],"-norandom")==0) ||
				(strcmp(argv[i],"-r")==0)
		) {

			userandom=0;
			knownparam=1;
		}
	       	if (
				(strcmp(argv[i],"-skipintro")==0) ||
				(strcmp(argv[i],"-s")==0)
		) {

			skipintro=1;
			knownparam=1;
		}
		if (!knownparam) {
			if ( (global_size_id<0) || (global_size_id>2) ) {
				print_help();
			} else {
				global_size_id=atoi(argv[i]);
			}
		}
	}

	if (SDL_Init (SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) < 0) {
		fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
		exit (1);
	}
	atexit (Terminate);

	SDL_ShowCursor(SDL_DISABLE);

	int flags = SDL_SWSURFACE;
#ifdef N800
	flags |= SDL_FULLSCREEN;
#endif

	screen = SDL_SetVideoMode (getScreenWidth(), getScreenHeight(), 16, flags);
	if (screen == NULL) {
		fprintf (stderr, "Couldn't set video mode: %s\n", SDL_GetError ());
		exit (2);
	}

	if (SDL_NumJoysticks() > 0) {
		joy = SDL_JoystickOpen(0);
		if(!joy) {
			fprintf (stderr, "Couldn't open joystick 0: %s\n", SDL_GetError ());
		}
	}

	if (TTF_Init() == -1) {
		exit(3);
	}

	initAudioSystem(useaudio);

	if (!skipintro) playIntro();

#ifdef LINUX
#endif

#ifdef GP2X
#endif
	// Music music(SHAREDIR"/data/music/mainmusic.ogg");
	Music menumusic(SHAREDIR"/data/music/menu.ogg");
	menumusic.play();
	// music.setVolume(globalMusicVolume);

	Game game;
	SDL_Event event;
	Uint32 score=0;

	vector<Image *> background;
	background.push_back(new Image(SHAREDIR"/data/gfx/"+intToString(getSizeID())+"/menu0.png"));
	background.push_back(new Image(SHAREDIR"/data/gfx/"+intToString(getSizeID())+"/menu1.png"));
	background.push_back(new Image(SHAREDIR"/data/gfx/"+intToString(getSizeID())+"/menu2.png"));
	background.push_back(new Image(SHAREDIR"/data/gfx/"+intToString(getSizeID())+"/menu3.png"));
	background.push_back(new Image(SHAREDIR"/data/gfx/"+intToString(getSizeID())+"/menu4.png"));
	int active=3;
	int fadeout=1;
	int fadeoffset=0;
	int keyVolDown=0;
	int keyVolUp=0;
	SDL_Rect fadeRect;
	fadeRect.w=getScreenWidth();
	fadeRect.x=0;
	fadeRect.y=0;
	Uint32 delayVal;
	int delay;

	list<string> *filter=new list<string>;
	filter->push_back(".ogg");
	filter->push_back(".mp3");
	FileList musicList=FileList(filter, SHAREDIR"/data/music");
	randomMusic=new FileListRandomizer(&musicList);

	while (SDL_PollEvent(&event)) {
	}

	while (!done) {
		while (SDL_PollEvent (&event)) {
			switch (event.type) {
				case SDL_KEYDOWN:
				case SDL_JOYBUTTONDOWN:
					if (
							( event.type==SDL_KEYDOWN && (event.key.keysym.sym == SDLK_c) && (event.key.keysym.mod & (KMOD_LCTRL | KMOD_RCTRL)) ) ||
							(event.type==SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE) ||
							( event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_SELECT )
					   ) {
						done = 1;
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym==SDLK_RETURN) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_START) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_FB)
					   ) {
						if (active==4) {
							config_dialog(&userandom, &numberOfRandomLevels);
						} else {
							SDL_Delay(25);
							fadeoffset=0;
							fadeRect.h=0;
							while (fadeout) {
								delayVal=SDL_GetTicks();
								sdlLock(screen);
								fadeRect.h=fadeoffset;
								SDL_FillRect(screen,&fadeRect,0);
								sdlUnlock(screen);
								SDL_Flip(screen);
								delay=25-(SDL_GetTicks()-delayVal);
								if (delay>0) SDL_Delay(delay);
								fadeoffset+=getScreenCoordinate(5);
								if (fadeoffset>background[active]->getHeight()) fadeout=0;
							}
							fadeout=1;

							menumusic.stop();
							menumusic.rewind();
							/* music.rewind();
							music.play(); */
							score=game.run(active,userandom,numberOfRandomLevels);
							// music.stop();
							menumusic.play();
						}
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym==SDLK_UP) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_UP)
					   ) {
						if (active>0) {
							active--;
						}
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym==SDLK_DOWN) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_DOWN)
					   ) {
						if ((unsigned int)active<(background.size()-1)) {
							active++;
						}
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym == SDLK_1) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_VOL_DOWN)
					   ) {
						keyVolDown=1;
					}
					if (
							(event.type==SDL_KEYDOWN && event.key.keysym.sym == SDLK_2) ||
							(event.type==SDL_JOYBUTTONDOWN && event.jbutton.button == VK_VOL_UP)
					   ) {
						keyVolUp=1;
					}
					break;
				case SDL_KEYUP:
				case SDL_JOYBUTTONUP:
					if (
							(event.type==SDL_KEYUP && event.key.keysym.sym == SDLK_1) ||
							(event.type==SDL_JOYBUTTONUP && event.jbutton.button == VK_VOL_DOWN)
					   ) {
						keyVolDown=0;
					}
					if (
							(event.type==SDL_KEYUP && event.key.keysym.sym == SDLK_2) ||
							(event.type==SDL_JOYBUTTONUP && event.jbutton.button == VK_VOL_UP)
					   ) {
						keyVolUp=0;
					}
					break;
				case SDL_QUIT:
					done = 1;
					break;
				default:
					break;
			}
		}

		if (keyVolDown) {
			globalMusicVolume--;
			if (globalMusicVolume<0) globalMusicVolume=0;
			setGlobalVolume(globalMusicVolume);
		}
		if (keyVolUp) {
			globalMusicVolume++;
			if (globalMusicVolume>128) globalMusicVolume=128;
			setGlobalVolume(globalMusicVolume);
		}
		sdlLock(screen);
		background[active]->blit(screen,0,0);	
		sdlUnlock(screen);
		SDL_Flip(screen);
		SDL_Delay(25);
	}

	menumusic.stop();

	return 0;
}
